# babel-plugin-import

Modular import plugin for babel, compatible with [antd](https://github.com/ant-design/ant-design), [antd-mobile](https://github.com/ant-design/ant-design-mobile), and so on.

[![NPM version](https://img.shields.io/npm/v/babel-plugin-import.svg?style=flat)](https://npmjs.org/package/babel-plugin-import)
[![Build Status](https://img.shields.io/travis/ant-design/babel-plugin-import.svg?style=flat)](https://travis-ci.org/ant-design/babel-plugin-import)

----

## CHANGELOG

1.0.0

Rename to babel-plugin-import.

- **BREAKCHANGE:** Don't support `libDir` anymore, replace it with `libraryDirectory`
- **BREAKCHANGE:** Remove `libraryName` default value (previous `antd`)
- [#66](https://github.com/ant-design/babel-plugin-antd/issues/66), Support material-ui
  - `libraryDirectory` could be empty string
  - add option `camel2DashComponentName`, default `true`
- [#67](https://github.com/ant-design/babel-plugin-antd/pull/67), Support ExpressionStatement

0.5.1

- [#50](https://github.com/ant-design/babel-plugin-antd/pull/50) - Support both antd and antd-mobile
- [#51](https://github.com/ant-design/babel-plugin-antd/pull/51) - Support export import

## Example

Converts

```javascript
import { Button } from 'antd';

ReactDOM.render(<div>
  <Button>xxxx</Button>
</div>);
```

(roughly) to

```javascript
var _button = require('antd/lib/button');

ReactDOM.render(<div>
  <_button>xxxx</_button>
</div>);
```

## Usage

```bash
npm install babel-plugin-import --save-dev
```

Via `.babelrc` or babel-loader.

```js
{
  "plugins": [["import", options]]
}
```

### options

`options` can be object.

```javascript
{
  "style": true,
  "libraryDirectory": "component",  // default: lib
  "libraryName": "antd",
  "camel2DashComponentName": false, // default: true
}
```

`options` can be an array.

For Example: 

```javascript
[
  {
    "libraryName": "antd",
    "libraryDirectory": "lib",   // default: lib
    "style": true
  },
  {
    "libraryName": "antd-mobile",
    "libraryDirectory": "component",
  },
]
```

### style

- `["import", [{ "libraryName": "antd" }]]`: import js modularly
- `["import", [{ "libraryName": "antd", "style": true }]]`: import js and css modularly (less source files)
- `["import", [{ "libraryName": "antd", "style": "css" }]]`: import js and css modularly (css built files)
